; *********************************************************** ;
;                           Skeleton                          ;
;       Make Timer 1 trigger an interrupt after 100ms	      ;
;                                                             ;
;       INFO0064 - Embedded Systems - Lab 3                   ;
;                                                             ;
; *********************************************************** ;

processor 16f1789
#include 	"config.inc"

; That is where the MCU will start executing the program (0x00)
	org 	0x00
	nop
	goto    start		    ; jump to the beginning of the code
	
; That is where the MCU will start executing the program (0x00)
	org 	0x04
	nop
	goto    interrupt_routine   ; jump to the interrupt routine

;BEGINNING OF THE PROGRAM
start:
	call	initialisation      ; initialisation routine configuring the MCU
	goto	main_loop           ; main loop

;INITIALISATION
initialisation:

    ; Configuration of clock - ? frequency - ? source
	movlb	0x01
	movlw	b'01101110'
	movwf	OSCCON		    ; configure oscillator (cf datasheet SFR OSCCON)
	movlw	b'00000000'	    
	movwf	OSCTUNE		    ; configure oscillator (cf datasheet SFR OSCTUNE)
    
    ; Timer1 ON - ?x prescaling - ? clock source
	movlb	0x00
	movlw	b'00110000'
	movwf	T1CON               ; configure Timer1 (cf. datasheet SFR T1CON)
	
    ; Interrupt configuration
	movlb	0x01
	bsf	PIE1,	TMR1IE	; enable timer 1 interruts
	bsf	INTCON,	GIE	; enable global interrupts
	bsf	INTCON,	6	; enable peripheral interrupts
	
    ; Start Timer 1 for 100ms period (12500 ticks)
	movlb	0x00
	movlw	0xCF
	movwf	TMR1H
	movlw	0x2B
	movwf	TMR1L
	bsf	T1CON, TMR1ON
	
	return
	
; Interrupt routine
interrupt_routine:
    movlb 0x00    
    btfss	PIR1, TMR1IF ; Test Timer 1 interrupt flag
	goto end_if_timer1
	
	bcf	PIR1, TMR1IF
	
	; Reset timer 1 for 12500 ticks
	bcf	T1CON, TMR1ON
	movlw	0xCF
	movwf	TMR1H
	movlw	0x2B
	movwf	TMR1L
	bsf	T1CON, TMR1ON

    end_if_timer1:
    
    retfie

;MAIN LOOP
main_loop:
    nop
    goto    main_loop
    END